# The LearningOnline Network with CAPA 
# definition of tags that give a structure to a document
#
# $Id: bridgetask.pm,v 1.42 2005/08/15 19:41:43 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
###


package Apache::bridgetask; 

use strict;
use Apache::lonnet;
use Apache::File();
use Apache::lonmenu;
use Apache::lonlocal;
use Apache::lonxml;
use Apache::slotrequest();
use Time::HiRes qw( gettimeofday tv_interval );

BEGIN {
    &Apache::lonxml::register('Apache::bridgetask',('Task','IntroParagraph','Dimension','Instance','InstanceText','Criteria','ClosingParagraph'));
}

sub initialize_bridgetask {
    # id of current Dimension, 0 means that no dimension is current 
    # (inside <Task> only)
    $Apache::bridgetask::dimension='';
    # list of all Dimension ids seen
    @Apache::bridgetask::dimensionlist=();
    # mandatory attribute of all Dimensions seen
    %Apache::bridgetask::dimensionmandatory=();
    # list of all current Instance ids
    @Apache::bridgetask::instance=();
    # list of all Instance ids seen in this problem
    @Apache::bridgetask::instancelist=();
    # key of queud user data that we are currently grading
    $Apache::bridgetask::queue_key='';
}

sub proctor_check_auth {
    my ($slot_name,$slot)=@_;
    my $user=$env{'form.proctorname'};
    my $domain=$env{'form.proctordomain'};
    
    my @allowed=split(",",$slot->{'proctor'});
    foreach my $possible (@allowed) {
	my ($puser,$pdom)=(split('@',$possible));
	if ($puser eq $user && $pdom eq $domain) {
	    my $authhost=&Apache::lonnet::authenticate($puser,$env{'form.proctorpassword'},$pdom);
	    if ($authhost ne 'no_host') {
		my $version=
		    $Apache::lonhomework::results{'resource.version'}=
		    ++$Apache::lonhomework::history{'resource.version'};
		$Apache::lonhomework::results{"resource.$version.checkedin"}=
		    $user.'@'.$domain;
		$Apache::lonhomework::results{"resource.$version.checkedin.slot"}=
		    $slot_name;
		foreach my $key (keys(%Apache::lonhomework::history)) {
		    if ($key=~/^resource\.0\./) {
			$Apache::lonhomework::results{$key}='';
		    }
		}
		return 1;
	    }
	}
    }
    return 0;
}

sub get_version {
    my ($version,$previous);
    if ($env{'form.previousversion'} && 
	$env{'form.previousversion'} ne 'current' &&
	defined($Apache::lonhomework::history{'resource.'.$env{'form.previousversion'}.'.status'})) {
	$version=$env{'form.previousversion'};
	$previous=1;
    } else {
	$version=$Apache::lonhomework::history{'resource.version'};
	$previous=0;
    }
    if (wantarray) {
	return ($version,$previous);
    }
    return $version;
}

sub add_previous_version_button {
    my ($status)=@_;
    my $result;
    if ($Apache::lonhomework::history{'resource.version'} eq '') {
	return '';
    }
    if ($Apache::lonhomework::history{'resource.version'} < 2 &&
	$status ne 'NEEDS_CHECKIN') {
	return '';
    }
    my $version=&get_version();
    if ($env{'form.previousversion'} ne '' &&
	$env{'form.previousversion'} eq $version) {
	$result.="<h3>".&mt("Showing previous version [_1]",$version).
	    "</h3>\n";
    }
    my @to_show;
    foreach my $test_version (1..$Apache::lonhomework::history{'resource.version'}) {
	if (defined($Apache::lonhomework::history{'resource.'.$test_version.'.status'})) {
	    push(@to_show,$test_version);
	}
    }
    my $list='<option>'.
	join("</option>\n<option>",@to_show).
	     "</option>\n";
    $list.='<option value="current">'.&mt('Current').'</option>';
    $result.='<form name="getprevious" method="POST" action="';
    my $uri=$env{'request.uri'};
    if ($env{'request.enc'}) { $uri=&Apache::lonenc::encrypted($uri); }
    $result.=$uri.'">'.
	&mt(' Show a previously done version: [_1]','<select onchange="this.form.submit()" name="previousversion">
<option>'.&mt('Pick one').'</option>
'.$list.'
</select>')."</form>";
    return $result;
}

sub add_grading_button {
    my $result=' <input type="submit" name="gradeasubmission" value="'.
	&mt("Get a submission to grade").'" />';
    $result.='<input type="hidden" name="grade_target" value="webgrade" />';
    if (&Apache::lonnet::allowed('mgq',$env{'request.course.id'})) {
	my ($entries,$ready,$locks)=&get_queue_counts('gradingqueue');
	$result.='<p>'.&mt("Grading Queue has [_1] entries. [_2] of them are ready to be graded and [_3] of them are currently being graded",$entries,$ready,$locks);

	$result.=' <input type="submit" name="reviewagrading" value="'.
	    &mt("Select an entry from the grading queue").'" /> </p>'."\n";

	($entries,$ready,$locks)=&get_queue_counts('reviewqueue');
	$result.='<p>'.&mt("Review Queue has [_1] entries. [_2] of them are ready to be graded and [_3] of them are currently being graded",$entries,$ready,$locks);
	$result.=' <input type="submit" name="reviewasubmission" value="'.
	    &mt("Select an entry from the review queue").'" /> </p>'."\n";
    }
    return $result;
}

sub add_request_another_attempt_button {
    my ($text)=@_;
    if (!$text) { $text="Request another attempt"; }
    my $result;
    my $symb=&Apache::lonnet::symbread();
    my ($slot_name,$slot)=&Apache::slotrequest::check_for_reservation($symb);
    my $action='get_reservation';
    if ($slot_name) {
	$text="Change reservation.";
	$action='change_reservation';
	my $description=&Apache::slotrequest::get_description($slot_name,
							      $slot);
	$result.=(<<STUFF);
<p> Will be next available: $description </p>
STUFF
    }
    
    if ($env{'request.enc'}) { $symb=&Apache::lonenc::encrypted($symb); }
    $symb=&Apache::lonnet::escape($symb);
    $result.='<form method="POST" action="/adm/slotrequest">'.
	'<input type="hidden" name="symb" value="'.$symb.'" />'.
	'<input type="hidden" name="command" value="'.$action.'" />'.
	'<input type="submit" name="requestattempt" value="'.
	&mt($text).'" />'.
	'</form>';
    return $result;
}

sub preserve_grade_info {
    my $result;
    # if we are viewing someone else preserve that info
    if (defined $env{'form.grade_symb'}) {
	foreach my $field ('symb','courseid','domain','username') {
	    $result .= '<input type="hidden" name="grade_'.$field.
		'" value="'.$env{"form.grade_$field"}.'" />'."\n";
	}
    }
    return $result;
}

sub start_Task {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;

    my ($status,$accessmsg,$slot);
    if ($target ne 'webgrade') {
	&Apache::structuretags::initialize_storage();
	&Apache::lonhomework::showhash(%Apache::lonhomework::history);
    } 

    $Apache::lonhomework::parsing_a_task=1;
    #should get back a <html> or the neccesary stuff to start XML/MathML
    my ($result,$head_tag_start,$body_tag_start,$form_tag_start)=
	&Apache::structuretags::page_start($target,$token,$tagstack,$parstack,$parser,$safeeval);
    
    if ($target eq 'web') {
	if ($Apache::lonhomework::modifygrades) {
	    $body_tag_start.='<form name="gradesubmission" method="POST" action="';
	    my $uri=$env{'request.uri'};
	    if ($env{'request.enc'}) { $uri=&Apache::lonenc::encrypted($uri); }
	    $body_tag_start.=$uri.'">'.&add_grading_button()."</form>";
	    my $symb=&Apache::lonnet::symbread();
	    if (&Apache::lonnet::allowed('mgq',$env{'request.course.id'})) {
		$body_tag_start.='<form method="POST" action="/adm/slotrequest">'.
		    '<input type="hidden" name="symb" value="'.$symb.'" />'.
		    '<input type="hidden" name="command" value="showslots" />'.
		    '<input type="submit" name="requestattempt" value="'.
		    &mt('Show Slot list').'" />'.
		    '</form>';
	    }
	}
    }
    if ($target eq 'web' || ($target eq 'grade' && !$env{'form.webgrade'}) || $target eq 'answer' ||
	$target eq 'tex') {
	my ($version,$previous)=&get_version();
	($status,$accessmsg,my $slot_name,$slot) = 
	    &Apache::lonhomework::check_task_access('0');
	push(@Apache::inputtags::status,$status);
	$Apache::inputtags::slot_name=$slot_name;
	my $expression='$external::datestatus="'.$status.'";';
	$expression.='$external::gradestatus="'.$Apache::lonhomework::history{"resource.$version.solved"}.'";';
	&Apache::run::run($expression,$safeeval);
	&Apache::lonxml::debug("Got $status");
	$body_tag_start.=&add_previous_version_button($status);
	if (!$previous && (
			   ( $status eq 'CLOSED' ) ||
			   ( $status eq 'BANNED') ||
			   ( $status eq 'UNAVAILABLE') ||
			   ( $status eq 'NOT_IN_A_SLOT') ||
			   ( $status eq 'NEEDS_CHECKIN') ||
			   ( $status eq 'WAITING_FOR_GRADE') ||
			   ( $status eq 'INVALID_ACCESS') )) {
	    my $bodytext=&Apache::lonxml::get_all_text("/task",$parser);
	    if ( $target eq "web" ) {
		$result.= $head_tag_start.'</head>'.$body_tag_start;
		my $msg;
		if ($status eq 'UNAVAILABLE') {
		    $msg.='<h1>'.&mt('Unable to determine if this resource is open due to network problems. Please try again later.').'</h1>';
		} elsif ($status eq 'NOT_IN_A_SLOT') {
		    $msg.='<h1>'.&mt('You are not currently signed up to work at this time and/or place.').'</h1>';
		    $msg.=&add_request_another_attempt_button("Sign up for time to work.");
		} elsif ($status eq 'NEEDS_CHECKIN') {
		    $msg.='<h1>'.&mt('You need the Proctor to validate you.').
			'</h1>'.&proctor_validation_screen($slot);
		} elsif ($status eq 'WAITING_FOR_GRADE') {
		    $msg.='<h1>'.&mt('Your submission is in the grading queue.').'</h1>';
		} elsif ($status ne 'NOT_YET_VIEWED') {
		    $msg.='<h1>'.&mt('Not open to be viewed').'</h1>';
		}
		if ($status eq 'CLOSED' || $status eq 'INVALID_ACCESS') {
		    $msg.='The problem '.$accessmsg;
		}
		$result.=$msg.'<br />';
	    } elsif ($target eq 'tex') {
		$result.='\begin{document}\noindent \vskip 1 mm  \begin{minipage}{\textwidth}\vskip 0 mm';
		if ($status eq 'UNAVAILABLE') {
		    $result.=&mt('Unable to determine if this resource is open due to network problems. Please try again later.').'\vskip 0 mm ';
		} else {
		    $result.=&mt('Problem is not open to be viewed. It')." $accessmsg \\vskip 0 mm ";
		}
	    } elsif ($target eq 'grade' && !$env{'form.webgrade'}) {
		if ($status eq 'NEEDS_CHECKIN') {
		    &proctor_check_auth($slot_name,$slot);
		}
	    }
	} elsif ($target eq 'web') {
	    my $name= &Apache::structuretags::get_resource_name($parstack,$safeeval);
	    $result.="$head_tag_start<title>$name</title></head>
              $body_tag_start \n";
	    
	    my ($version,$previous)=&get_version();
	    if ($Apache::lonhomework::history{"resource.$version.status"} eq 'fail') {
		$result.='<h1><font color="red">'.&mt('Did not pass').'</font></h1>';
		if (!$previous) {
		    $result.=&add_request_another_attempt_button();
		}
	    }
	    if ($Apache::lonhomework::history{"resource.$version.status"} eq 'pass') {
		$result.='<h1><font color="green">'.&mt('Passed').'</font></h1>';
	    }
	    $result.=$form_tag_start.
		'<input type="hidden" name="submitted" value="yes" />';
	    $result.=&preserve_grade_info();

	}
    } elsif ( ($target eq 'grade' && $env{'form.webgrade'}) ||
	      $target eq 'webgrade') {
	my $webgrade='yes';
	if ($target eq 'webgrade') {
	    $result.=$head_tag_start.$body_tag_start;
	    $result.='<br />Review'.&show_queue('reviewqueue');
	    $result.='<br />Grade'.&show_queue('gradingqueue');
	}
	# FIXME Blast! still need to reorg this, need to reshow the
        #       queue being reviewed once done with the grade pass...
        #       Hrrm, vaildation pass should perhaps say 'not_locked'
        #       perhaps do a search if there is a key that is mine and if
        #       there isn't reshow the queue....
	my ($todo,$status_code)=&get_key_todo($target);

	&Apache::lonnet::logthis("got todo -$todo- stat -$status_code-");
	if ($todo) {
	    &setup_env_for_other_user($todo,$safeeval);
	    my ($symb,$uname,$udom)=&decode_queue_key($todo);
	    $result.="\n".'<table><tr><td>Found '.
		&Apache::lonnet::gettitle($symb).' for '.$uname.' at '.$udom.'</td></tr></table>';
	    $form_tag_start.=
		'<input type="hidden" name="gradingkey" value="'.
		&Apache::lonnet::escape($todo).'" />';
	    $Apache::bridgetask::queue_key=$todo;
	    &Apache::structuretags::initialize_storage();
	    &Apache::lonhomework::showhash(%Apache::lonhomework::history);
	    if ($target eq 'webgrade') {
		$result.='<br />After -'.&show_queue($env{'form.queue'});
		$result.="\n".'<table width="100%" style="width:100%" border="1">';
		if ($status_code eq 'selected') {
		    $form_tag_start.=
			'<input type="hidden" name="queuemode" value="selected" />';
		}
	    }
	} else {
	    if ($target eq 'webgrade') {
		$result.="\n";
		if      ($status_code eq 'stop') {
		    $result.='<b>'.&mt("Stopped grading.").'</b>';
		} elsif ($status_code eq 'lock_failed') {
		    $result.='<b>'.&mt("Failed to lock the request record.").'</b>';
		} elsif ($status_code eq 'unlock') {
		    $result.='<b>'.&mt("Unlocked the requested record.").'</b>';
		    $result.=&show_queue($env{'form.queue'},1);
		} elsif ($status_code eq 'show_list') {
		    $result.=&show_queue($env{'form.queue'},1);
		} else {
		    $result.='<b>'.&mt("No user to be graded.").'</b>';
		}
	    }
	    $webgrade='no';
	    my $bodytext=&Apache::lonxml::get_all_text("/task",$parser);
	}
	if ($target eq 'webgrade' && defined($env{'form.queue'})) {
	    $result.=$form_tag_start;
	    $result.='<input type="hidden" name="webgrade" value="'.
		$webgrade.'" />';
	    $result.='<input type="hidden" name="queue" value="'.
		$env{'form.queue'}.'" />';
	}
    } else {
	# page_start returned a starting result, delete it if we don't need it
	$result = '';
    }
    return $result;
}

sub get_key_todo {
    my ($target)=@_;
    my $todo;

    if (defined($env{'form.reviewasubmission'})) {
	&Apache::lonnet::logthis("review a submission....");
	$env{'form.queue'}='reviewqueue';
	return (undef,'show_list');
    }

    if (defined($env{'form.reviewagrading'})) {
	&Apache::lonxml::debug("review a grading....");
	$env{'form.queue'}='gradingqueue';
	return (undef,'show_list');
    }

    my $queue=$env{'form.queue'};

    if (!defined($queue)) {
	$env{'form.queue'}=$queue='gradingqueue';
    }

    my $gradingkey=&Apache::lonnet::unescape($env{'form.gradingkey'});

    if (defined($env{'form.queue'}) && defined($env{'form.gradingkey'})
	&& !defined($env{'form.gradingaction'}) 
	&& $env{'form.queuemode'} eq 'selected') {
	
	my $who=&queue_key_locked($queue,$gradingkey);
	my $me=$env{'user.name'}.'@'.$env{'user.domain'};
	if ($who eq $me) {
	    &Apache::lonxml::debug("Found a key was given to me");
	    return ($gradingkey,'selected');
	} else {
	    return (undef,'show_list');
	}

    }

    if ($target eq 'webgrade' && $env{'form.queuemode'} eq 'selected') {
	if ($env{'form.gradingaction'} eq 'resume') {
	    delete($env{'form.gradingaction'});
	    &Apache::lonxml::debug("Resuming a key");
	    return ($gradingkey);
	} elsif ($env{'form.gradingaction'} eq 'unlock') {
	    &Apache::lonxml::debug("Unlocking a key ".
				     &check_queue_unlock($queue,$gradingkey,1));
	    return (undef,'unlock');
	} elsif ($env{'form.gradingaction'} eq 'select') {
	    &Apache::lonxml::debug("Locking a key");
	    if (&lock_key($queue,$gradingkey)) {
		&Apache::lonxml::debug("Success $queue");
		return ($gradingkey);
	    }
	    &Apache::lonxml::debug("Failed $queue");
	    return (undef,'lock_failed');
	}
    }

    if ($env{'form.queuemode'} ne 'selected') {
	# don't get something new from the queue if they hit the stop button
    	if (!($env{'form.stop'} && $target eq 'webgrade') 
	    && !$env{'form.gradingaction'}) {
	    &Apache::lonxml::debug("Getting anew $queue");
	    return (&get_from_queue($queue));
	} else {
	    return (undef,'stop');
	}
    }
    return (undef,undef)
}

sub end_Task {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result='';
    my $status=$Apache::inputtags::status['-1'];
    my ($version,$previous)=&get_version();
    if ($target eq 'grade' || $target eq 'web' || $target eq 'answer' ||
	$target eq 'tex') {
	if (
	    (($target eq 'web') && ($env{'request.state'} ne 'construct')) ||
	    ($target eq 'answer') || ($target eq 'tex')
	   ) {
	    if ($target eq 'web') {
		if ($status eq 'CAN_ANSWER' && !$previous) {
		    $result.="\n".'<table border="1">'.
			&Apache::inputtags::file_selector("$version.0",
							  "bridgetask","*",
							  'portfolioonly').
							  "</table>";
		    $result.=&Apache::inputtags::gradestatus('0');
		}
	    } 
	    if ($target eq 'web' || $target eq 'webgrade') {
		$result.=&Apache::lonxml::xmlend().'</html>';
	    }
	}
	if ($target eq 'grade' && !$env{'form.webgrade'} && !$previous) {
	    my $award='SUBMITTED';
	    &Apache::essayresponse::file_submission("$version.0",'bridgetask',
						    'portfiles',\$award);
	    if ($award eq 'SUBMITTED' &&
		$Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"}) {
		$Apache::lonhomework::results{"resource.0.tries"}=
		    $Apache::lonhomework::results{"resource.$version.0.tries"}=
		    1+$Apache::lonhomework::history{"resource.$version.0.tries"};

		$Apache::lonhomework::results{"resource.0.award"}=
		    $Apache::lonhomework::results{"resource.$version.0.award"}=
		    $award;
	    }
	    &Apache::lonhomework::showhash(%Apache::lonhomework::results);
	    &Apache::structuretags::finalize_storage();
	    if ($award eq 'SUBMITTED') {
		&add_to_queue('gradingqueue',$Apache::inputtags::slot_name);
	    }
	}
	if ($target eq 'grade' && $env{'form.webgrade'} eq 'yes') {
	    my $optional_required=
		&Apache::lonxml::get_param('OptionalRequired',$parstack,
					   $safeeval);
	    my $optional_passed=0;
	    my $mandatory_failed=0;
	    my $ungraded=0;
	    my $review=0;   
	    &Apache::lonhomework::showhash(%Apache::lonhomework::results);
	    foreach my $dim_id (@Apache::bridgetask::dimensionlist) {
		my $status=
		    $Apache::lonhomework::results{"resource.$version.$dim_id.status"};
		my $mandatory=
		    ($Apache::bridgetask::dimensionmandatory{$dim_id} ne 'N');
		if ($status eq 'pass') {
		    if (!$mandatory) { $optional_passed++; }
		} elsif ($status eq 'fail') {
		    if ($mandatory) { $mandatory_failed++; }
		} elsif ($status eq 'ungraded') {
		    $ungraded++;
		} elsif ($status eq 'review') {
		    $review++;
		}	
	    }
	    if ($optional_passed < $optional_required) {
		$mandatory_failed++;
	    }
	    &Apache::lonxml::debug("all dim ".join(':',@Apache::bridgetask::dimensionlist)."results -> m_f $mandatory_failed o_p $optional_passed u $ungraded r $review");
	    $Apache::lonhomework::results{'resource.grader'}=
		$env{'user.name'}.'@'.$env{'user.domain'};
	    if ($review) {
		$Apache::lonhomework::results{"resource.$version.status"}='review';
		if ($env{'form.queue'} eq 'reviewqueue') {
		    &check_queue_unlock($env{'form.queue'});
		    &Apache::lonxml::debug(" still needs review not changing status.");
		} else {
		    &move_between_queues('gradingqueue','reviewqueue');
		}
	    } elsif ($ungraded) {
		$Apache::lonhomework::results{"resource.$version.status"}='ungraded';
		if ($env{'form.queue'} eq 'reviewqueue') {
		    &Apache::lonxml::debug("moving back.");
		    &move_between_queues('reviewqueue','gradingqueue');
		} else {
		    &check_queue_unlock($env{'form.queue'});
		}
	    } elsif ($mandatory_failed) {
		$Apache::lonhomework::results{"resource.$version.status"}='fail';
		$Apache::lonhomework::results{"resource.$version.0.solved"}='incorrect_by_override';
		$Apache::lonhomework::results{"resource.$version.0.award"}='INCORRECT';
		$Apache::lonhomework::results{"resource.$version.0.awarded"}='0';
		&remove_from_queue($env{'form.queue'}); 

		my ($symb,$courseid,$udom,$uname)=&Apache::lonxml::whichuser();
		$Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"}=
		    $Apache::lonhomework::history{"resource.$version.0.bridgetask.portfiles"};
		&Apache::grades::version_portfiles(
					\%Apache::lonhomework::results,
					 ["$version.0.bridgetask"],$courseid,
					 $symb,$udom,$uname,
					 ["$version.0.bridgetask"]);
	    } else {
		$Apache::lonhomework::results{"resource.$version.status"}='pass';
		$Apache::lonhomework::results{"resource.$version.0.solved"}='correct_by_override';
		$Apache::lonhomework::results{"resource.$version.0.award"}='EXACT_ANS';
		$Apache::lonhomework::results{"resource.$version.0.awarded"}='1';
		&remove_from_queue($env{'form.queue'});

		my ($symb,$courseid,$udom,$uname)=&Apache::lonxml::whichuser();
		$Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"}=
		    $Apache::lonhomework::history{"resource.$version.0.bridgetask.portfiles"};
		&Apache::grades::version_portfiles(
					\%Apache::lonhomework::results,
					 ["$version.0.bridgetask"],$courseid,
					 $symb,$udom,$uname,
					 ["$version.0.bridgetask"]);
	    }
	    $Apache::lonhomework::results{"resource.status"}=
		$Apache::lonhomework::results{"resource.$version.status"};
	    if (defined($Apache::lonhomework::results{"resource.$version.0.awarded"})) {
		$Apache::lonhomework::results{"resource.0.award"}=
		    $Apache::lonhomework::results{"resource.$version.award"};
		$Apache::lonhomework::results{"resource.0.awarded"}=
		    $Apache::lonhomework::results{"resource.$version.awarded"};
		$Apache::lonhomework::results{"resource.0.solved"}=
		    $Apache::lonhomework::results{"resource.$version.solved"};
	    }
	    &Apache::structuretags::finalize_storage();
	}
    } elsif ($target eq 'webgrade') {
	$result.="</table>\n<hr />";
	#$result.='<input type="submit" name="next" value="'.
	#    &mt('Save &amp; Next').'" /> ';
	#$result.='<input type="submit" name="end" value="'.
	#    &mt('Save &amp; Stop Grading').'" /> ';
	#$result.='<input type="submit" name="throwaway" value="'.
	#    &mt('Throw Away &amp; Stop Grading').'" /> ';
	#$result.='<input type="submit" name="save" value="'.
	#    &mt('Save Partial Grade and Continue Grading').'" /> ';
	$result.='</form>'.&Apache::loncommon::endbodytag().'</html>';
    } elsif ($target eq 'meta') {
	$result.='<parameter part="0" package="Task"></parameter>'."\n";
	#$result.=&Apache::response::meta_part_order();
	#$result.=&Apache::response::meta_response_order();
    }
    undef($Apache::lonhomework::parsing_a_task);
    return $result;
}

sub move_between_queues {
    my ($src_queue,$dest_queue)=@_;
    my $cur_data=&get_queue_data($src_queue);
    if (!$cur_data) { return 'not_exist'; }
    my $result=&add_to_queue($dest_queue,$cur_data->[0]);
    if ($result ne 'ok') {
	return $result;
    }
    &check_queue_unlock($src_queue);
    return &remove_from_queue($src_queue);
}

sub check_queue_unlock {
    my ($queue,$key,$allow_not_me)=@_;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    if (!defined($key)) {
	$key="$symb\0queue\0$uname\@$udom";
    }
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $me=$env{'user.name'}.'@'.$env{'user.domain'};
    my $who=&queue_key_locked($queue,$key,$cdom,$cnum);
    if  ($who eq $me) {
	return &Apache::lonnet::del($queue,["$key\0locked"],$cdom,$cnum);
    } elsif ($allow_not_me) {
	&Apache::lonxml::debug("unlocking $who by $me");
	return &Apache::lonnet::del($queue,["$key\0locked"],$cdom,$cnum);
    }
    return 'not_owner';
}

sub remove_from_queue {
    my ($queue)=@_;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $key="$symb\0queue\0$uname\@$udom";
    my @keys=($key,"$key\0locked");
    return &Apache::lonnet::del($queue,\@keys,$cdom,$cnum);
}

sub setup_env_for_other_user {
    my ($queue_key,$safeeval)=@_;
    my ($symb,$uname,$udom)=&decode_queue_key($queue_key);
    &Apache::lonxml::debug("setup_env for $queue_key");
    $env{'form.grade_symb'}=$symb;
    $env{'form.grade_domain'}=$udom;
    $env{'form.grade_username'}=$uname;
    $env{'form.grade_courseid'}=$env{'request.course.id'};
    &Apache::lonxml::initialize_rndseed($safeeval);
}

sub get_queue_data {
    my ($queue)=@_;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $todo="$symb\0queue\0$uname\@$udom";
    my ($key,$value)=&Apache::lonnet::get($queue,[$todo],$cdom,$cnum);
    if ($key eq $todo && ref($value)) {
	return $value;
    }
    return undef;
}

sub add_to_queue {
    my ($queue,$slot_name)=@_;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my %data;
    $data{"$symb\0queue\0$uname\@$udom"}=[$slot_name];
    return &Apache::lonnet::put($queue,\%data,$cdom,$cnum);
}

sub show_queue {
    my ($queue,$with_selects)=@_;
    my $result;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $regexp="^$symb\0";
    my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    my ($tmp)=%queue;
    if ($tmp=~/^error: 2 /) {
	return "\n<h3>Current Queue - $queue</h3><table border='1'><tr><td>Empty</td></tr></table>";
    }
    $result.="\n<h3>Current Queue - $queue </h3><table border='1'><tr>";
    if ($with_selects) { $result.="<th></th>"; }
    $result.="<th>resource</th><th>user</th><th>type</th><th>data</th></tr>";
    foreach my $key (sort(keys(%queue))) {
	if ($key=~/locked$/ && !$with_selects) {
	    my ($symb,$uname,$udom) = &decode_queue_key($key);
	    my $title=&Apache::lonnet::gettitle($symb);
	    $result.="<tr><td>$title</td><td>$uname</td>";
	    $result.='<td>lock</td><td>'.$queue{$key}.'</td></tr>';
	} elsif ($key=~/timestamp$/ && !$with_selects) {
	    my ($symb,undef) = split("\0",$key);
	    my $title=&Apache::lonnet::gettitle($symb);
	    $result.="<tr><td>$title</td><td></td>";
	    $result.='<td>last queue modification time</td><td>'.
		&Apache::lonlocal::locallocaltime($queue{$key})."</td></tr>";
	} elsif ($key!~/(timestamp|locked)$/) {
	    my ($symb,$uname,$udom) = &decode_queue_key($key);
	    my $title=&Apache::lonnet::gettitle($symb);
	    $result.="<tr>";
	    my $slot=$queue{$key}->[0];
	    my %slot_data=&Apache::lonnet::get_slot($slot);
	    if ($with_selects) {
		my $ekey=&Apache::lonnet::escape($key);
		my ($action,$description)=('select',&mt('Select'));
		if (exists($queue{"$key\0locked"})) {
		    my $me=$env{'user.name'}.'@'.$env{'user.domain'};
		    if ($me eq $queue{"$key\0locked"}) {
			($action,$description)=('resume',&mt('Resume'));
		    } else {
			($action,$description)=('unlock',&mt('Unlock'));
		    }
		}
		if (time > $slot_data{'endtime'}) {
		    $result.=(<<FORM);
<td>
<form method="POST">
 <input type="hidden" name="gradingkey" value="$ekey" />
 <input type="hidden" name="queue" value="$queue" />
 <input type="hidden" name="gradingaction" value="$action" />
 <input type="hidden" name="webgrade" value="no" />
 <input type="hidden" name="queuemode" value="selected" />
 <input type="submit" name="submit" value="$description" />
</form>
</td>
FORM

                } else {
		    $result.='<td>'.&mt("In Progress").'</td>'
		}
	    }
	    $result.="<td>$title</td><td>$uname</td>";
	    $result.='<td>queue entry</td><td>Slot: '.$slot.' End time: '.
		&Apache::lonlocal::locallocaltime($slot_data{'endtime'}).
		"</td></tr>";
	}
    }
    $result.="</table><hr />\n";
    return $result;
}

sub get_queue_counts {
    my ($queue)=@_;
    my $result;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $regexp="^$symb\0";
    my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    my ($tmp)=%queue;
    if ($tmp=~/^error: 2 /) {
	return (0,0,0);
    }
    my ($entries,$ready_to_grade,$locks)=(0,0,0);
    foreach my $key (sort(keys(%queue))) {
	if ($key=~/locked$/) {
	    $locks++;
	} elsif ($key=~/timestamp$/) {
	    #ignore
	} elsif ($key!~/(timestamp|locked)$/) {
	    my ($symb,$uname,$udom) = &decode_queue_key($key);
	    $entries++;
	    my $slot=$queue{$key}->[0];
	    my %slot_data=&Apache::lonnet::get_slot($slot);
	    if (time > $slot_data{'endtime'}) {
		$ready_to_grade++;
	    }
	}
    }
    return ($entries,$ready_to_grade,$locks);
}

sub decode_queue_key {
    my ($key)=@_;
    my ($symb,undef,$user) = split("\0",$key);
    my ($uname,$udom) = split('@',$user);
    return ($symb,$uname,$udom);
}

sub queue_key_locked {
    my ($queue,$key,$cdom,$cnum)=@_;
    if (!defined($cdom) || !defined($cnum)) {
	my (undef,$cid)=&Apache::lonxml::whichuser();
	$cnum=$env{'course.'.$cid.'.num'};
	$cdom=$env{'course.'.$cid.'.domain'};
    }
    my ($key_locked,$value)=
	&Apache::lonnet::get($queue,["$key\0locked"],$cdom,$cnum);
    if ($key_locked eq "$key\0locked") {
	return $value;
    }
    return undef;
}

sub pick_from_queue_data {
    my ($queue,$check_section,$queuedata,$cdom,$cnum)=@_;
    foreach my $key (keys(%$queuedata)) {
	my ($symb,$uname,$udom)=&decode_queue_key($key);
	if ($check_section) {
	    my $section=&Apache::lonnet::getsection($uname,$udom);
	    if ($section eq $check_section) {
		&Apache::lonxml::debug("my sec");
		next;
	    }
	}
	my $slot=$queuedata->{$key}[0];
	my %slot_data=&Apache::lonnet::get_slot($slot);
	if ($slot_data{'endtime'} > time) { 
	    &Apache::lonxml::debug("not time");
	    next;
	}
	if (&queue_key_locked($queue,$key,$cdom,$cnum)) {
	    &Apache::lonxml::debug("someone already has um.");
	    next;
	}
	return $key;
    }
    return undef;
}

sub find_mid_grade {
    my ($queue,$symb,$cdom,$cnum)=@_;
    my $todo=&Apache::lonnet::unescape($env{'form.gradingkey'});
    my $me=$env{'user.name'}.'@'.$env{'user.domain'};
    if ($todo) {
	my $who=&queue_key_locked($queue,$todo,$cdom,$cnum);
	if ($who eq $me) { return $todo; }
    }
    my $regexp="^$symb\0.*\0locked\$";
    my %locks=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    foreach my $key (keys(%locks)) {
	my $who=$locks{$key};
	if ($who eq $me) {
	    $todo=$key;
	    $todo=~s/\0locked$//;
	    return $todo;
	}
    }
    return undef;
}

sub lock_key {
    my ($queue,$todo)=@_;
    my $me=$env{'user.name'}.'@'.$env{'user.domain'};
    my (undef,$cid)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $success=&Apache::lonnet::newput($queue,{"$todo\0locked"=> $me},
					$cdom,$cnum);
    &Apache::lonxml::debug("success $success $todo");
    if ($success eq 'ok') {
	return 1;
    }
    return 0;
}

sub get_from_queue {
    my ($queue)=@_;
    my $result;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $todo=&find_mid_grade($queue,$symb,$cdom,$cnum);
    &Apache::lonxml::debug("found ".join(':',&decode_queue_key($todo)));
    if ($todo) { return $todo; }
    while (1) {
	my $starttime=time;
	&Apache::lonnet::put($queue,{"$symb\0timestamp"=>$starttime},
			     $cdom,$cnum);
	&Apache::lonxml::debug("$starttime");
	my $regexp="^$symb\0queue\0";
	my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
	#make a pass looking for a user _not_ in my section
	if ($env{'request.course.sec'}) {
	    &Apache::lonxml::debug("sce");
	    $todo=&pick_from_queue_data($queue,$env{'request.course.sec'},
					\%queue,$cdom,$cnum);
	    &Apache::lonxml::debug("sce $todo");
	}
	# no one _not_ in our section so look for any user that is
	# ready for grading
	if (!$todo) {
	    &Apache::lonxml::debug("no sce");
	    $todo=&pick_from_queue_data($queue,$env{'request.course.sec'},
					\%queue,$cdom,$cnum);
	    &Apache::lonxml::debug("no sce $todo");
	}
	# no user to grade 
	if (!$todo) { last; }
	&Apache::lonxml::debug("got $todo");
	# otherwise found someone so lets try to lock them
	# unless someone else already picked them
	if (!&lock_key($queue,$todo)) { next; }
	my (undef,$endtime)=
	    &Apache::lonnet::get($queue,["$symb\0timestamp"],
				 $cdom,$cnum);
	&Apache::lonxml::debug("emd  $endtime");
	# someone else already modified the queue, 
	# perhaps our picked user wass already fully graded between
	# when we picked him and when we locked his record? so lets
	# double check.
	if ($endtime != $starttime) {
	    my ($key,$value)=
		&Apache::lonnet::get($queue,["$todo"],
				     $cdom,$cnum);
	    &Apache::lonxml::debug("check  $key .. $value");
	    if ($key eq $todo && ref($value)) {
	    } else {
		&Apache::lonnet::del($queue,["$todo\0locked"],
				     $cdom,$cnum);
		&Apache::lonxml::debug("del");
		next;
	    }
	}
	&Apache::lonxml::debug("last $todo");
	last;
    }
    return $todo;
}

sub start_ClosingParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result;
    if ($target eq 'web') {
    } elsif ($target eq 'webgrade') {
	&Apache::lonxml::startredirection();
    }
    return $result;
}

sub end_ClosingParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result;
    if ($target eq 'web') {
    } elsif ($target eq 'webgrade') {
	&Apache::lonxml::endredirection();
    }
    return $result;
}

sub get_id {
    my ($parstack,$safeeval)=@_;
    my $id=&Apache::lonxml::get_param('id',$parstack,$safeeval);
    if (!$id) { $id=$Apache::lonxml::curdepth; }
    return $id;
}

my %dimension;
sub start_Dimension {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    undef(%dimension);
    my $dim_id=&get_id($parstack,$safeeval);
    $Apache::bridgetask::dimension=$dim_id;
    push(@Apache::bridgetask::dimensionlist,$dim_id);
    undef(@Apache::bridgetask::instance);
    $Apache::bridgetask::dimensionmandatory{$dim_id}=
	&Apache::lonxml::get_param('Mandatory',$parstack,$safeeval);
    return '';
}

sub get_instance {
    &Apache::response::pushrandomnumber();
    my @order=&Math::Random::random_permutation(@{$dimension{'instances'}});
    my $num=@order;
    my $version=&get_version();
    my $which=($version-1)%$num;
    return $order[$which];
}

{
    my $last_link;
    sub end_Dimension {
	my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
	my $result;
	my $dim=&get_id($parstack,$safeeval);
	my $instance=&get_instance();
	my $version=&get_version();
	if ($target eq 'web') {
	    $result=$dimension{'intro'}.$dimension{$instance.'.text'};
	    if ($Apache::lonhomework::history{"resource.$version.status"} eq 'pass' ||
		$Apache::lonhomework::history{"resource.$version.status"} eq 'fail') {
		foreach my $id (@{$dimension{$instance.'.criterias'}}) {
		    my $status=$Apache::lonhomework::history{"resource.$version.$dim.$instance.$id.status"};
		    my $comment=$Apache::lonhomework::history{"resource.$version.$dim.$instance.$id.comment"};
		    $result.='<table border="1"><tr><td>';
		    if ($status eq 'fail') {
			$result.='<font color="red">'.&mt('Did not pass').'</font>';
		    } elsif ($status eq 'pass') {
			$result.='<font color="green">'.&mt('Passed').'</font>';
		    } else {
			&Apache::lonxml::error("Student viewing a graded bridgetask was show a status of $status");
		    }
		    $result.='</td></tr><tr><td>';
		    if ($Apache::lonhomework::history{"resource.$version.$dim.$instance.$id.comment"}) {
			$result.=&mt('Comment from grader: ').$Apache::lonhomework::history{"resource.$version.$dim.$instance.$id.comment"}.'</td></tr><tr><td>';
		    }
		    $result.=$dimension{$instance.'.criteria.'.$id}.
			'</td></tr></table>';
		}
	    }
	} elsif ($target eq 'webgrade') {
	    foreach my $id (@{$dimension{$instance.'.criterias'}}) {
		my $link='criteria_'.$instance.'_'.$id;
		my $status=$Apache::lonhomework::history{"resource.$version.$dim.$instance.$id.status"};
		$result.='<tr><td width="100%" valign="top">'.
		    '<a name="'.$link.'" />'.
		    '<a name="next_'.$last_link.'" />'.
		    '<br /><textarea enabled="false" style="width:100%" rows="8" width="25" wrap="hard">'.$dimension{$instance.'.criteria.'.$id}.'</textarea>'.
		    #$dimension{$instance.'.criteria.'.$id}.
		    '</td>'.
		    '<td><nobr>Additional Comment for Student</nobr> <br />'.
		    '<textarea style="width:100%" rows="8" width="25" wrap="hard" name="HWVAL_comment_'.$link.'">'.&HTML::Entities::encode($Apache::lonhomework::history{"resource.$version.$dim.$instance.$id.comment"}).'</textarea>'.
		    '</td>'.
		    '<td>'.
		    '<nobr><label><input type="radio" name="HWVAL_'.$link.'" value="ungraded" '.($status eq 'ungraded' || !$status ? 'checked="checked"':'').'/>'.&mt('Ungraded').'</label></nobr><br />'.
		    '<label><input type="radio" name="HWVAL_'.$link.'" value="pass" '.($status eq 'pass' ? 'checked="checked"':'').' />'.&mt('Pass').'</label><br />'.
		    '<label><input type="radio" name="HWVAL_'.$link.'" value="fail" '.($status eq 'fail' ? 'checked="checked"':'').' />'.&mt('Fail').'</label><br />'.
		    '<label><input type="radio" name="HWVAL_'.$link.'" value="review" '.($status eq 'review' ? 'checked="checked"':'').' />'.&mt('Review').'</label><br />'.
		    '</td>'.
		    '<td>'.
		    '<a href="#'.$last_link.'">Prev</a><br />'.
		    '<a href="#next_'.$link.'">Next</a><br /><br /><br />'.
		    '<input type="submit" name="next" value="'.
		    &mt('Done').'" /> '.
		    '<input type="submit" name="stop" value="'.
		    &mt('Stop').'" /> '.
		    '</td></tr>';
		$result.='<tr><td colspan="4">';
		my (undef,undef,$udom,$uname) = &Apache::lonxml::whichuser();
		my $file_url = '/uploaded/'.$udom.'/'.$uname.'/portfolio';
		foreach my $partial_file (split(',',$Apache::lonhomework::history{"resource.$version.bridgetask.portfiles"})) {
		    my $file=$file_url.$partial_file;
		    &Apache::lonnet::allowuploaded('/adm/bridgetask',$file);
		    $result.='<a href="'.$file.'" target="lonGRDs"><img src="'.
			&Apache::loncommon::icon($file).'" border=0"> '.$file.
			'</a>';
		}
		$result.='</td></tr>';
		$last_link=$link;
	    }
	} elsif ($target eq 'grade' && $env{'form.webgrade'}) {
	    my $optional_passed=0;
	    my $mandatory_failed=0;
	    my $ungraded=0;
	    my $review=0;
	    foreach my $id (@{$dimension{$instance.'.criterias'}}) {
		my $status=$Apache::lonhomework::results{"resource.$version.$dim.$instance.$id.status"}=$env{'form.HWVAL_criteria_'.$instance.'_'.$id};
		$Apache::lonhomework::results{"resource.$version.$dim.$instance.$id.comment"}=$env{'form.HWVAL_comment_criteria_'.$instance.'_'.$id};
		my $mandatory=($dimension{$instance.'.criteria.'.$id.'.mandatory'} ne 'N');
		if ($status eq 'pass') {
		    if (!$mandatory) { $optional_passed++; }
		} elsif ($status eq 'fail') {
		    if ($mandatory) { $mandatory_failed++; }
		} elsif ($status eq 'review') {
		    $review++;
		} elsif ($status eq 'ungraded') {
		    $ungraded++;
		} else {
		    &Apache::lonxml::error("got weird status --$status--");
		}
	    }
	    if ($optional_passed < $dimension{$instance.'.optionalrequired'}) {
		$mandatory_failed++;
	    }
	    &Apache::lonxml::debug("all instance ".join(':',@{$dimension{$instance.'.criterias'}})." results -> m_f $mandatory_failed o_p $optional_passed u $ungraded r $review");
	    if ($review) {
		$Apache::lonhomework::results{"resource.$version.$dim.status"}=
		    'review';
	    } elsif ($ungraded) {
		$Apache::lonhomework::results{"resource.$version.$dim.status"}=
		    'ungraded';
	    } elsif ($mandatory_failed) {
		$Apache::lonhomework::results{"resource.$version.$dim.status"}=
		    'fail';
	    } else {
		$Apache::lonhomework::results{"resource.$version.$dim.status"}=
		    'pass';
	    }
	}
	return $result;
    }
}

sub start_IntroParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result;
    if ($target eq 'web' || $target eq 'webgrade') {
	if ($tagstack->[-2] eq 'Dimension' || $target eq 'webgrade') {
	    &Apache::lonxml::startredirection();
	}
    }
    return $result;
}

sub end_IntroParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result;
    if ($target eq 'web' || $target eq 'webgrade') {
	if ($tagstack->[-2] eq 'Dimension' || $target eq 'webgrade') {
	    $dimension{'intro'}=&Apache::lonxml::endredirection();
	}
    }
    return $result;
}

sub start_Instance {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $id=&get_id($parstack,$safeeval);
    push(@{$dimension{'instances'}},$id);
    push(@Apache::bridgetask::instance,$id);
    push(@Apache::bridgetask::instancelist,$id);
    $dimension{$id.'.optionalrequired'}=
	&Apache::lonxml::get_param('OptionalRequired',$parstack,$safeeval);
    return '';
}

sub end_Instance {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    return '';
}

sub start_InstanceText {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    if ($target eq 'web' || $target eq 'webgrade') {
	&Apache::lonxml::startredirection();
    }
    return '';
}

sub end_InstanceText {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $instance_id=$Apache::bridgetask::instance[-1];
    if ($target eq 'web' || $target eq 'webgrade') {
	$dimension{$instance_id.'.text'}=&Apache::lonxml::endredirection();
    }
    return '';
}

sub start_Criteria {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    if ($target eq 'web' || $target eq 'webgrade' || $target eq 'grade') {
	&Apache::lonxml::startredirection();
    }
    return '';
}

sub end_Criteria {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $instance_id=$Apache::bridgetask::instance[-1];
    if ($target eq 'web' || $target eq 'webgrade' || $target eq 'grade') {
	my $criteria=&Apache::lonxml::endredirection();
	my $id=&get_id($parstack,$safeeval);
	$dimension{$instance_id.'.criteria.'.$id}=$criteria;
	$dimension{$instance_id.'.criteria.'.$id.'.mandatory'}=
	    &Apache::lonxml::get_param('Mandatory',$parstack,$safeeval);
	push(@{$dimension{$instance_id.'.criterias'}},$id);
    }
    return '';
}

sub proctor_validation_screen {
    my ($slot) = @_;
    my (undef,undef,$domain,$user) = &Apache::lonxml::whichuser();
    my $url=&Apache::lonnet::studentphoto($domain,$user,'jpg');
    $user=$env{'form.proctorname'};
    if ($env{'form.proctordomain'}) { $domain=$env{'form.proctordomain'}; }
    my $msg;
    if ($env{'form.proctorpassword'}) {
	$msg='<p><font color="red">'.&mt("Failed to authenticate the proctor.")
	    .'</font></p>';
    }
    my $result= (<<ENDCHECKOUT);
<h2>Proctor Validation</h2>
    <p>Your room's proctor needs to validate your access to this resource.</p>
    $msg
<form name="checkout" method="POST" action="$env{'request.uri'}">
<input type="hidden" name="validate" value="yes" />
<input type="hidden" name="submitted" value="yes" />
<table>
  <tr><td>Proctor's Username:</td><td><input type="string" name="proctorname" value="$user" /></td></tr>
  <tr><td>Password:</td><td><input type="password" name="proctorpassword" value="" /></td></tr>
  <tr><td>Proctor's Domain:</td><td><input type="string" name="proctordomain" value="$domain" /></td></tr>
</table>
<input type="submit" name="checkoutbutton" value="Validate"  /><br />
Student who should be logged in is:<br />
<img src="$url" /><br />
</form>
ENDCHECKOUT
    return $result;
}

1;
__END__
